/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn;

import net.fabricmc.api.ClientModInitializer;
import net.fabricmc.fabric.api.client.model.ModelLoadingRegistry;
import net.minecraft.class_1058;
import net.minecraft.class_1087;
import net.minecraft.class_1088;
import net.minecraft.class_1091;
import net.minecraft.class_1100;
import net.minecraft.class_1799;
import net.minecraft.class_2378;
import net.minecraft.class_2960;
import net.minecraft.class_3611;
import net.minecraft.class_3612;
import net.minecraft.class_3665;
import reborncore.client.gui.builder.GuiBase;
import reborncore.client.hud.StackInfoHUD;
import techreborn.client.render.DynamicBucketBakedModel;
import techreborn.client.render.DynamicCellBakedModel;
import techreborn.events.StackToolTipHandler;
import techreborn.init.TRContent;
import techreborn.items.ItemDynamicCell;
import techreborn.items.ItemFrequencyTransmitter;
import techreborn.utils.StackWIPHandler;

import java.util.Collection;
import java.util.Collections;
import java.util.Set;
import java.util.function.Function;

public class TechRebornClient implements ClientModInitializer {

	@Override
	public void onInitializeClient() {
		ModelLoadingRegistry.INSTANCE.registerAppender((manager, out) -> {
			out.accept(new class_1091(new class_2960(TechReborn.MOD_ID, "cell_base"), "inventory"));
			out.accept(new class_1091(new class_2960(TechReborn.MOD_ID, "cell_fluid"), "inventory"));
			out.accept(new class_1091(new class_2960(TechReborn.MOD_ID, "cell_background"), "inventory"));
			out.accept(new class_1091(new class_2960(TechReborn.MOD_ID, "cell_glass"), "inventory"));

			out.accept(new class_1091(new class_2960(TechReborn.MOD_ID, "bucket_base"), "inventory"));
			out.accept(new class_1091(new class_2960(TechReborn.MOD_ID, "bucket_fluid"), "inventory"));
			out.accept(new class_1091(new class_2960(TechReborn.MOD_ID, "bucket_background"), "inventory"));
		});

		ModelLoadingRegistry.INSTANCE.registerVariantProvider(resourceManager -> (modelIdentifier, modelProviderContext) -> {
			if (modelIdentifier.method_12836().equals(TechReborn.MOD_ID)) {
				if (modelIdentifier.method_12832().equals("cell")) {
					return new class_1100() {
						@Override
						public Collection<class_2960> method_4755() {
							return Collections.emptyList();
						}

						@Override
						public Collection<class_2960> method_4754(Function<class_2960, class_1100> function, Set<String> set) {
							return Collections.emptyList();
						}

						@Override
						public class_1087 method_4753(class_1088 modelLoader, Function<class_2960, class_1058> function, class_3665 modelBakeSettings) {
							return new DynamicCellBakedModel();
						}
					};
				}
				class_3611 fluid = class_2378.field_11154.method_10223(new class_2960(TechReborn.MOD_ID, modelIdentifier.method_12832().split("_bucket")[0]));
				if (modelIdentifier.method_12832().endsWith("_bucket") && fluid != class_3612.field_15906) {
					return new class_1100() {
						@Override
						public Collection<class_2960> method_4755() {
							return Collections.emptyList();
						}

						@Override
						public Collection<class_2960> method_4754(Function<class_2960, class_1100> function, Set<String> set) {
							return Collections.emptyList();
						}

						@Override
						public class_1087 method_4753(class_1088 modelLoader, Function<class_2960, class_1058> function, class_3665 modelBakeSettings) {
							return new DynamicBucketBakedModel(fluid);
						}
					};
				}
			}
			return null;
		});

		StackToolTipHandler.setup();
		StackWIPHandler.setup();

		GuiBase.wrenchStack = new class_1799(TRContent.WRENCH);
		GuiBase.fluidCellProvider = ItemDynamicCell::getCellWithFluid;

		StackInfoHUD.registerElement(new ItemFrequencyTransmitter.StackInfoFreqTransmitter());
	}


}
