/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.cable;

import reborncore.api.IListInfoProvider;
import reborncore.api.IToolDrop;
import reborncore.api.power.EnergyBlockEntity;
import reborncore.api.power.EnumPowerTier;
import reborncore.common.powerSystem.PowerSystem;
import reborncore.common.util.StringUtils;
import techreborn.blocks.cable.BlockCable;
import techreborn.init.TRBlockEntities;
import techreborn.init.TRContent;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import net.minecraft.class_124;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2248;
import net.minecraft.class_2350;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2586;
import net.minecraft.class_2622;
import net.minecraft.class_3000;

/**
 * Created by modmuss50 on 19/05/2017.
 */

public class CableBlockEntity extends class_2586
	implements class_3000, IListInfoProvider, IToolDrop, EnergyBlockEntity {
	
	private double energy = 0;
	private int transferRate = 0;
	private TRContent.Cables cableType = null;
	private ArrayList<class_2350> sendingFace = new ArrayList<class_2350>();
	int ticksSinceLastChange = 0;

	public CableBlockEntity() {
		super(TRBlockEntities.CABLE);
	}

	private TRContent.Cables getCableType() {
		class_2248 block = field_11863.method_8320(field_11867).method_11614();
		if(block instanceof BlockCable){
			return ((BlockCable) block).type;
		}
		//Something has gone wrong if this happens
		return TRContent.Cables.COPPER;
	}

	@Override
    public class_2487 method_16887() {
        return method_11007(new class_2487());
    }

    @Override
    public class_2622 method_16886() {
        class_2487 nbtTag = new class_2487();
        method_11007(nbtTag);
        return new class_2622(method_11016(), 1, nbtTag);
    }


    @Override
    public void method_11014(class_2487 compound) {
        super.method_11014(compound);
        if (compound.method_10545("CableBlockEntity")) {
            energy = compound.method_10562("CableBlockEntity").method_10550("power");
        }
    }

    @Override
    public class_2487 method_11007(class_2487 compound) {
        super.method_11007(compound);
        if (energy > 0) {
        	class_2487 data = new class_2487();
    		compound.method_10566("CableBlockEntity", data);
        }
        return compound;
    }
	
	// ITickable
	@Override
	public void method_16896() {
		if (field_11863.field_9236) {
			return;
		}

		if (cableType == null) {
			cableType = getCableType();
			transferRate = cableType.transferRate;
		}

		ticksSinceLastChange++;
		if (ticksSinceLastChange >= 20) {
			sendingFace.clear();
			ticksSinceLastChange = 0;
		}

		ArrayList<EnergyBlockEntity> acceptors = new ArrayList<EnergyBlockEntity>();
		for (class_2350 face : class_2350.values()) {
			class_2586 blockEntity = field_11863.method_8321(field_11867.method_10093(face));

			if (blockEntity == null) {
				continue;
			} else if (blockEntity instanceof EnergyBlockEntity) {
				EnergyBlockEntity acceptor = (EnergyBlockEntity) blockEntity;
				if (blockEntity instanceof CableBlockEntity && energy <= acceptor.getEnergy()) {
					continue;
				}
				if (!acceptor.canAcceptEnergy(face.method_10153())) {
					continue;
				}
				acceptors.add(acceptor);
				if (!sendingFace.contains(face)) {
					sendingFace.add(face);
				}
			}
		}

		if (acceptors.size() == 0) {
			return;
		}
		Collections.shuffle(acceptors);
        acceptors.forEach(blockEntity -> {
            double drain = Math.min(energy, transferRate);
            if (drain > 0 && blockEntity.addEnergy(drain, true) > 0) {
                double move = blockEntity.addEnergy(drain, false);
                useEnergy(move, false);
            }
        });
	}

    // IListInfoProvider
	@Override
	public void addInfo(List<class_2561> info, boolean isReal, boolean hasData) {
		if (isReal) {
			info.add(new class_2585(class_124.field_1080 + StringUtils.t("techreborn.tooltip.transferRate") + ": "
				+ class_124.field_1065
				+ PowerSystem.getLocaliszedPowerFormatted(transferRate) + "/t"));
			info.add(new class_2585(class_124.field_1080 + StringUtils.t("techreborn.tooltip.tier") + ": "
				+ class_124.field_1065 + StringUtils.toFirstCapitalAllLowercase(cableType.tier.toString())));
		}
	}

	// IToolDrop
	@Override
	public class_1799 getToolDrop(class_1657 playerIn) {
		return new class_1799(getCableType().block);
	}

	@Override
	public double getEnergy() {
		return energy;
	}

	@Override
	public void setEnergy(double energy) {
		this.energy = energy;
	}

	@Override
	public double getMaxPower() {
		return transferRate * 4;
	}

	@Override
	public boolean canAddEnergy(double energyIn) {
		return getEnergy() + energyIn <= getMaxPower();
	}

	@Override
	public double addEnergy(double energy) {
		return addEnergy(energy, false);
	}

	@Override
	public double addEnergy(double energyIn, boolean simulate) {
		double energyReceived = Math.min(getMaxPower(), Math.min(getMaxPower() - getEnergy(), energyIn));

		if (!simulate) {
			setEnergy(getEnergy() + energyReceived);
		}
		return energyReceived;
	}

	@Override
	public boolean canUseEnergy(double energy) {
		return this.energy >= energy;
	}

	@Override
	public double useEnergy(double energy) {
		return useEnergy(energy, false);
	}

	@Override
	public double useEnergy(double energyOut, boolean simulate) {
		if (energyOut > energy) {
			energyOut = energy;
		}
		if (!simulate) {
			setEnergy(energy - energyOut);
		}
		return energyOut;
	}

	@Override
	public boolean canAcceptEnergy(class_2350 direction) {
		if (sendingFace.contains(direction)) {
			return false;
		}
		return getMaxPower() != getEnergy();
	}

	@Override
	public boolean canProvideEnergy(class_2350 direction) {
		return true;
	}

	@Override
	public double getMaxOutput() {
		return transferRate;
	}

	@Override
	public double getMaxInput() {
		return transferRate;
	}

	@Override
	public EnumPowerTier getTier() {
		return EnumPowerTier.getTier(cableType.transferRate);
	}
}
