/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blocks.cable;

import net.minecraft.block.*;
import net.minecraft.class_1268;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1921;
import net.minecraft.class_1922;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2237;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2398;
import net.minecraft.class_2464;
import net.minecraft.class_2586;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689;
import net.minecraft.class_2733;
import net.minecraft.class_2746;
import net.minecraft.class_3419;
import net.minecraft.class_3614;
import net.minecraft.class_3726;
import net.minecraft.class_3965;
import reborncore.api.ToolManager;
import reborncore.api.power.EnergyBlockEntity;
import reborncore.common.blocks.BlockWrenchEventHandler;
import reborncore.common.util.WrenchUtils;
import techreborn.config.TechRebornConfig;
import techreborn.init.ModSounds;
import techreborn.init.TRContent;
import techreborn.blockentity.cable.CableBlockEntity;
import techreborn.utils.damageSources.ElectrialShockSource;

import javax.annotation.Nullable;

/**
 * Created by modmuss50 on 19/05/2017.
 */
public class BlockCable extends class_2237 {

	public static final class_2746 EAST = class_2746.method_11825("east");
	public static final class_2746 WEST = class_2746.method_11825("west");
	public static final class_2746 NORTH = class_2746.method_11825("north");
	public static final class_2746 SOUTH = class_2746.method_11825("south");
	public static final class_2746 UP = class_2746.method_11825("up");
	public static final class_2746 DOWN = class_2746.method_11825("down");

	public final TRContent.Cables type;

	public BlockCable(TRContent.Cables type) {
		super(class_2248.class_2251.method_9637(class_3614.field_15914).method_9629(1f, 8f));
		this.type = type;
		method_9590(this.field_10647.method_11664().method_11657(EAST, false).method_11657(WEST, false).method_11657(NORTH, false)
				.method_11657(SOUTH, false).method_11657(UP, false).method_11657(DOWN, false));
		BlockWrenchEventHandler.wrenableBlocks.add(this);
	}

	public class_2733<Boolean> getProperty(class_2350 facing) {
		switch (facing) {
		case field_11034:
			return EAST;
		case field_11039:
			return WEST;
		case field_11043:
			return NORTH;
		case field_11035:
			return SOUTH;
		case field_11036:
			return UP;
		case field_11033:
			return DOWN;
		default:
			return EAST;
		}
	}

	private class_2680 makeConnections(class_1937 world, class_2338 pos) {
		Boolean down = canConnectTo(world, pos.method_10074(), class_2350.field_11036);
		Boolean up = canConnectTo(world, pos.method_10084(), class_2350.field_11033);
		Boolean north = canConnectTo(world, pos.method_10095(), class_2350.field_11035);
		Boolean east = canConnectTo(world, pos.method_10078(), class_2350.field_11039);
		Boolean south = canConnectTo(world, pos.method_10072(), class_2350.field_11043);
		Boolean west = canConnectTo(world, pos.method_10067(), class_2350.field_11039);

		return this.method_9564().method_11657(DOWN, down).method_11657(UP, up).method_11657(NORTH, north).method_11657(EAST, east)
				.method_11657(SOUTH, south).method_11657(WEST, west);
	}

	private Boolean canConnectTo(class_1936 world, class_2338 pos, class_2350 facing) {
		class_2586 blockEntity = world.method_8321(pos);
		if (blockEntity != null && (blockEntity instanceof EnergyBlockEntity || blockEntity instanceof CableBlockEntity)) {
			return Boolean.TRUE;
		}
		return Boolean.FALSE;
	}

	// BlockContainer
	@Override
	public class_2464 method_9604(class_2680 state) {
		return class_2464.field_11458;
	}

	@Nullable
	@Override
	public class_2586 method_10123(class_1922 worldIn) {
		return new CableBlockEntity();
	}

	// Block
	@Override
	public boolean method_9534(class_2680 state, class_1937 worldIn, class_2338 pos, class_1657 playerIn, class_1268 hand, class_3965 hitResult) {
		class_1799 stack = playerIn.method_5998(class_1268.field_5808);
		class_2586 blockEntity = worldIn.method_8321(pos);

		// We should always have blockEntity entity. I hope.
		if (blockEntity == null) {
			return false;
		}

		if (!stack.method_7960() && ToolManager.INSTANCE.canHandleTool(stack)) {
			if (WrenchUtils.handleWrench(stack, worldIn, pos, playerIn, hitResult.method_17780())) {
				return true;
			}
		}
		return super.method_9534(state, worldIn, pos, playerIn, hand, hitResult);
	}

	@Override
	protected void method_9515(class_2689.class_2690<class_2248, class_2680> builder) {
		builder.method_11667(EAST, WEST, NORTH, SOUTH, UP, DOWN);
	}

	@Override
	public class_2680 method_9605(class_1750 context) {
		return makeConnections(context.method_8045(), context.method_8037());
	}

	@Override
	public class_2680 method_9559(class_2680 ourState, class_2350 ourFacing, class_2680 otherState,
			class_1936 worldIn, class_2338 ourPos, class_2338 otherPos) {
		Boolean value = canConnectTo(worldIn, otherPos, ourFacing.method_10153());
		return ourState.method_11657(getProperty(ourFacing), value);
	}

	@Override
	public class_1921 method_9551() {
		return class_1921.field_9174;
	}
	
	@Override
	public class_265 method_9530(class_2680 state, class_1922 world, class_2338 pos, class_3726 entityContext) {
		if (type != null) {
			// outline is 1px bigger then model to ease selection
			double culling = type.cableThickness - 1;
			return class_2248.method_9541(culling, culling, culling, 16.0D - culling, 16.0D - culling,
					16.0D - culling);
		}
		return class_2248.method_9541(4, 4, 4, 12, 12, 12);
	}

	@Override
	public void method_9548(class_2680 state, class_1937 worldIn, class_2338 pos, class_1297 entityIn) {
		super.method_9548(state, worldIn, pos, entityIn);
		if (!type.canKill) {
			return;
		}
		if (!(entityIn instanceof class_1309)) {
			return;
		}

		class_2586 blockEntity = worldIn.method_8321(pos);
		if (blockEntity == null) {
			return;
		}
		if (!(blockEntity instanceof CableBlockEntity)) {
			return;
		}

		CableBlockEntity blockEntityCable = (CableBlockEntity) blockEntity;
		if (blockEntityCable.getEnergy() <= 0) {
			return;
		}

		if (TechRebornConfig.uninsulatedElectrocutionDamage) {
			if (type == TRContent.Cables.HV) {
				entityIn.method_5639(1);
			}
			entityIn.method_5643(new ElectrialShockSource(), 1F);
		}
		if (TechRebornConfig.uninsulatedElectrocutionSound) {
			worldIn.method_8465(null, entityIn.field_5987, entityIn.field_6010, entityIn.field_6035, ModSounds.CABLE_SHOCK, class_3419.field_15245,
					0.6F, 1F);
		}
		if (TechRebornConfig.uninsulatedElectrocutionParticles) {
			worldIn.method_8406(class_2398.field_11205, entityIn.field_5987, entityIn.field_6010, entityIn.field_6035, 0, 0, 0);
		}
	}
}
