/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blocks.lighting;

import reborncore.api.ToolManager;
import reborncore.common.BaseBlockEntityProvider;
import reborncore.common.blocks.BlockWrenchEventHandler;
import reborncore.common.util.WrenchUtils;
import techreborn.blockentity.lighting.LampBlockEntity;

import javax.annotation.Nullable;
import net.minecraft.class_1268;
import net.minecraft.class_1657;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1921;
import net.minecraft.class_1922;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2464;
import net.minecraft.class_2586;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689;
import net.minecraft.class_2741;
import net.minecraft.class_2746;
import net.minecraft.class_2753;
import net.minecraft.class_3614;
import net.minecraft.class_3965;

public class BlockLamp extends BaseBlockEntityProvider {

	public static class_2753 FACING = class_2741.field_12525;
	public static class_2746 ACTIVE;
	protected final class_265[] shape;

	private int cost;
	private int brightness;

	public BlockLamp(int brightness, int cost, double depth, double width) {
		super(class_2248.class_2251.method_9637(class_3614.field_15918).method_9629(2f, 2f));
		this.method_9590(this.field_10647.method_11664().method_11657(FACING, class_2350.field_11043).method_11657(ACTIVE, false));
		this.shape = GenCuboidShapes(depth, width);
		this.cost = cost;
		this.brightness = brightness;
		BlockWrenchEventHandler.wrenableBlocks.add(this);
	}
	
	private class_265[] GenCuboidShapes(double depth, double width) {
		double culling = (16.0D - width) / 2 ;
		class_265[] shapes = {
				class_2248.method_9541(culling, 16.0 - depth, culling, 16.0 - culling, 16.0D, 16.0 - culling),
				class_2248.method_9541(culling, 0.0D, culling, 16.0D - culling, depth, 16.0 - culling),
				class_2248.method_9541(culling, culling, 16.0 - depth, 16.0 - culling, 16.0 - culling, 16.0D),
				class_2248.method_9541(culling, culling, 0.0D, 16.0 - culling, 16.0 - culling, depth),
				class_2248.method_9541(16.0 - depth, culling, culling, 16.0D, 16.0 - culling, 16.0 - culling),
				class_2248.method_9541(0.0D, culling, culling, depth, 16.0 - culling, 16.0 - culling)
			};
		return shapes;
	}
	
	public static boolean isActive(class_2680 state) {
		return state.method_11654(ACTIVE);
	}

	public static class_2350 getFacing(class_2680 state) {
		return state.method_11654(FACING);
	}

	public static void setFacing(class_2350 facing, class_1937 world, class_2338 pos) {
		world.method_8501(pos, world.method_8320(pos).method_11657(FACING, facing));
	}

	public static void setActive(Boolean active, class_1937 world, class_2338 pos) {
		class_2350 facing = (class_2350)world.method_8320(pos).method_11654(FACING);
		class_2680 state = world.method_8320(pos).method_11657(ACTIVE, active).method_11657(FACING, facing);
		world.method_8652(pos, state, 3);
	}

	public int getCost() {
		return cost;
	}
	
	// BaseTileBlock
	@Override
	public class_2586 method_10123(class_1922 worldIn) {
		return new LampBlockEntity();
	}	

	// Block
	@Override
	protected void method_9515(class_2689.class_2690<class_2248, class_2680> builder) {
		ACTIVE = class_2746.method_11825("active");
		builder.method_11667(FACING, ACTIVE);
	}
	
	@Nullable
	@Override
	public class_2680 method_9605(class_1750 context) {
		for (class_2350 enumfacing : context.method_7718()) {
			class_2680 iblockstate = this.method_9564().method_11657(FACING, enumfacing.method_10153());
			if (iblockstate.method_11591(context.method_8045(), context.method_8037())) {
				return iblockstate;
			}
		}
		return null;
	}
	
	@Override
	public int method_9593(class_2680 state) {
		return state.method_11654(ACTIVE) ? brightness : 0;
	}

	@Override
	public class_1921 method_9551() {
		return class_1921.field_9174;
	}
	
	@Override
	public class_2464 method_9604(class_2680 state) {
		return class_2464.field_11458;
	}
	
	@Override
	public boolean method_9534(class_2680 state, class_1937 worldIn, class_2338 pos, class_1657 playerIn, class_1268 hand, class_3965 hitResult) {
		class_1799 stack = playerIn.method_5998(class_1268.field_5808);
		class_2586 blockEntity = worldIn.method_8321(pos);

		// We extended BaseTileBlock. Thus we should always have blockEntity entity. I hope.
		if (blockEntity == null) {
			return false;
		}

		if (!stack.method_7960() && ToolManager.INSTANCE.canHandleTool(stack)) {
			if (WrenchUtils.handleWrench(stack, worldIn, pos, playerIn, hitResult.method_17780())) {
				return true;
			}
		}

		return super.method_9534(state, worldIn, pos, playerIn, hand, hitResult);
	}
}
