/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.client.gui;

import reborncore.common.util.StringUtils;
import net.minecraft.class_156;
import net.minecraft.class_1657;
import net.minecraft.class_2585;
import net.minecraft.class_2960;
import net.minecraft.class_407;
import net.minecraft.class_437;
import reborncore.client.gui.builder.widget.GuiButtonExtended;
import reborncore.common.network.NetworkManager;
import techreborn.config.TechRebornConfig;
import techreborn.items.ItemManual;
import techreborn.packets.ServerboundPackets;

public class GuiManual extends class_437 {

	ItemManual manual;
	class_1657 player;

	private static final class_2960 texture = new class_2960("techreborn", "textures/gui/manual.png");
	int guiWidth = 207;
	int guiHeight = 195;
	private static final String text1 = StringUtils.t("techreborn.manual.wiki");
	private static final String text2 = StringUtils.t("techreborn.manual.discord");
	private static final String text3 = StringUtils.t("techreborn.manual.refund");

	public GuiManual(class_1657 player) {
		super(new class_2585("gui.manual"));
		this.player = player;
	}

	@Override
	public void init() {
		int y = (height / 2) - guiHeight / 2;
		y+= 40;
		addButton(new GuiButtonExtended((width / 2 - 30), y + 10, 60, 20, StringUtils.t("techreborn.manual.wikibtn"), var1 -> minecraft.method_1507(new class_407(t -> {
			if(t){
				class_156.method_668().method_670("http://wiki.techreborn.ovh");
				this.minecraft.method_1507(this);
			} else {
				this.minecraft.method_1507(this);
			}
		}, "http://wiki.techreborn.ovh", false))));
		addButton(new GuiButtonExtended((width / 2 - 30), y + 60, 60, 20, StringUtils.t("techreborn.manual.discordbtn"), var1 -> minecraft.method_1507(new class_407(t -> {
			if(t){
				class_156.method_668().method_670("https://discord.gg/teamreborn");
				this.minecraft.method_1507(this);
			}else {
				this.minecraft.method_1507(this);
			}
		}, "https://discord.gg/teamreborn", false))));
		if(TechRebornConfig.allowManualRefund){
			addButton(new GuiButtonExtended((width / 2 - 30), y + 110, 60, 20, StringUtils.t("techreborn.manual.refundbtn"), var1 -> {
				NetworkManager.sendToServer(ServerboundPackets.createRefundPacket());
				this.minecraft.method_1507(null);
			}));
		}
	}

	@Override
	public void render(int mouseX, int mouseY, float partialTicks) {
		renderBackground();
		minecraft.method_1531().method_4618(GuiManual.texture);
		int centerX = (width / 2) - guiWidth / 2;
		int centerY = (height / 2) - guiHeight / 2;
		blit(centerX, centerY, 0, 0, guiWidth, guiHeight);
		font.method_1729(text1, ((width / 2) - font.method_1727(text1) / 2), centerY + 40, 4210752);
		font.method_1729(text2, ((width / 2) - font.method_1727(text2) / 2), centerY + 90, 4210752);
		if (TechRebornConfig.allowManualRefund) {
			font.method_1729(text3, ((width / 2) - font.method_1727(text3) / 2), centerY + 140, 4210752);
		}
		super.render(mouseX, mouseY, partialTicks);
	}
}


