/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.client.render;

import net.fabricmc.fabric.api.client.render.fluid.v1.FluidRenderHandler;
import net.fabricmc.fabric.api.client.render.fluid.v1.FluidRenderHandlerRegistry;
import net.fabricmc.fabric.api.renderer.v1.mesh.MutableQuadView;
import net.fabricmc.fabric.api.renderer.v1.mesh.QuadEmitter;
import net.fabricmc.fabric.api.renderer.v1.model.FabricBakedModel;
import net.fabricmc.fabric.api.renderer.v1.render.RenderContext;
import net.fabricmc.indigo.renderer.helper.GeometryHelper;
import net.minecraft.class_1058;
import net.minecraft.class_1087;
import net.minecraft.class_1091;
import net.minecraft.class_1092;
import net.minecraft.class_1799;
import net.minecraft.class_1920;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2680;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_3611;
import net.minecraft.class_3612;
import net.minecraft.class_777;
import net.minecraft.class_806;
import net.minecraft.class_809;
import reborncore.common.fluid.container.ItemFluidInfo;
import reborncore.common.util.Color;
import techreborn.TechReborn;

import javax.annotation.Nullable;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import java.util.function.Supplier;

public class DynamicBucketBakedModel implements class_1087, FabricBakedModel {

	private final class_3611 fluid;

	private static final class_1091 BUCKET_BASE = new class_1091(new class_2960(TechReborn.MOD_ID, "bucket_base"), "inventory");
	private static final class_1091 BUCKET_BACKGROUND = new class_1091(new class_2960(TechReborn.MOD_ID, "bucket_background"), "inventory");
	private static final class_1091 BUCKET_FLUID = new class_1091(new class_2960(TechReborn.MOD_ID, "bucket_fluid"), "inventory");

	public DynamicBucketBakedModel(class_3611 fluid) {
		this.fluid = fluid;
	}

	@Override
	public void emitBlockQuads(class_1920 blockView, class_2680 state, class_2338 pos, Supplier<Random> randomSupplier, RenderContext context) {

	}

	@Override
	public void emitItemQuads(class_1799 stack, Supplier<Random> randomSupplier, RenderContext context) {
		class_3611 fluid = class_3612.field_15906;
		if (stack.method_7909() instanceof ItemFluidInfo) {
			ItemFluidInfo fluidInfo = (ItemFluidInfo) stack.method_7909();
			fluid = fluidInfo.getFluid(stack);

		}
		class_1092 bakedModelManager = class_310.method_1551().method_1554();
		context.fallbackConsumer().accept(bakedModelManager.method_4742(BUCKET_BASE));
		context.fallbackConsumer().accept(bakedModelManager.method_4742(BUCKET_BACKGROUND));

		if (fluid != class_3612.field_15906) {
			FluidRenderHandler fluidRenderHandler = FluidRenderHandlerRegistry.INSTANCE.get(fluid);
			class_1087 fluidModel = bakedModelManager.method_4742(BUCKET_FLUID);
			int fluidColor = fluidRenderHandler.getFluidColor(class_310.method_1551().field_1687, class_310.method_1551().field_1724.method_5704(), fluid.method_15785());
			class_1058 fluidSprite = fluidRenderHandler.getFluidSprites(class_310.method_1551().field_1687, class_2338.field_10980, fluid.method_15785())[0];
			int color = new Color((float) (fluidColor >> 16 & 255) / 255.0F, (float) (fluidColor >> 8 & 255) / 255.0F, (float) (fluidColor & 255) / 255.0F).getColor();
			context.pushTransform(quad -> {
				quad.nominalFace(GeometryHelper.lightFace(quad));
				quad.spriteColor(0, color, color, color, color);
				quad.spriteBake(0, fluidSprite, MutableQuadView.BAKE_LOCK_UV);
				return true;
			});
			final QuadEmitter emitter = context.getEmitter();
			fluidModel.method_4707(null, null, randomSupplier.get()).forEach(q -> {
				emitter.fromVanilla(q.method_3357(), 0, false);
				emitter.emit();
			});
			context.popTransform();
		}
	}

	@Override
	public List<class_777> method_4707(@Nullable class_2680 blockState, @Nullable class_2350 direction, Random random) {
		return Collections.emptyList();
	}

	@Override
	public boolean isVanillaAdapter() {
		return false;
	}

	@Override
	public boolean method_4708() {
		return true;
	}

	@Override
	public boolean method_4712() {
		return false;
	}

	@Override
	public boolean method_4713() {
		return false;
	}

	@Override
	public class_1058 method_4711() {
		return class_310.method_1551().method_1549().method_4608(new class_2960("minecraft:item/bucket"));
	}

	@Override
	public class_809 method_4709() {
		return ModelHelper.DEFAULT_ITEM_TRANSFORMS;
	}

	@Override
	public class_806 method_4710() {
		return class_806.field_4292;
	}
}
