/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items.tool.industrial;

import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import reborncore.common.util.StringUtils;
import net.minecraft.class_124;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1271;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1761;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1800;
import net.minecraft.class_1829;
import net.minecraft.class_1834;
import net.minecraft.class_1836;
import net.minecraft.class_1937;
import net.minecraft.class_2371;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2588;
import net.minecraft.class_2960;
import net.minecraft.item.*;
import net.minecraft.util.*;
import reborncore.api.power.IEnergyItemInfo;
import reborncore.api.power.ItemPowerManager;
import reborncore.common.powerSystem.ExternalPowerSystems;
import reborncore.common.powerSystem.PowerSystem;
import reborncore.common.util.ChatUtils;
import reborncore.common.util.ItemDurabilityExtensions;
import reborncore.common.util.ItemUtils;
import techreborn.TechReborn;
import techreborn.config.TechRebornConfig;
import techreborn.init.TRContent;
import techreborn.utils.MessageIDs;

import javax.annotation.Nullable;
import java.util.List;

public class ItemNanosaber extends class_1829 implements IEnergyItemInfo, ItemDurabilityExtensions {
	public static final int maxCharge = TechRebornConfig.nanoSaberCharge;
	public int transferLimit = 1_000;
	public int cost = 250;

	// 4M FE max charge with 1k charge rate
	public ItemNanosaber() {
		super(class_1834.field_8930, 1, 1, new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP).method_7889(1));
		this.method_7863(new class_2960("techreborn:active"), new class_1800() {
			@Override
			@Environment(EnvType.CLIENT)
			public float call(class_1799 stack,
			                   @Nullable
				                   class_1937 worldIn,
			                   @Nullable
				                   class_1309 entityIn) {
				if (ItemUtils.isActive(stack)) {
					ItemPowerManager capEnergy = new ItemPowerManager(stack);
					if (capEnergy.getMaxEnergyStored() - capEnergy.getEnergyStored() >= 0.9	* capEnergy.getMaxEnergyStored()) {
						return 0.5F;
					}
					return 1.0F;
				}
				return 0.0F;
			}
		});
	}
	
	// ItemSword
	@Override
	public boolean method_7873(class_1799 stack, class_1309 entityHit, class_1309 entityHitter) {
		ItemPowerManager capEnergy = new ItemPowerManager(stack);
		if (capEnergy.getEnergyStored() >= cost) {
			capEnergy.useEnergy(cost, false);
			ExternalPowerSystems.requestEnergyFromArmor(capEnergy, entityHitter);

			return true;
		} else {
			return false;
		}
	}

	//TODO needs a stack aware version added in
	// Item
//	@Override
//	public Multimap<String, EntityAttributeModifier> getAttributeModifiers(EquipmentSlot slot) {
//		Multimap<String, EntityAttributeModifier> multimap = HashMultimap.<String, EntityAttributeModifier>create();
//		int modifier = 0;
//		if (ItemUtils.isActive(stack)) {
//			modifier = 9;
//		}
//		if (slot == EquipmentSlot.MAINHAND) {
//			multimap.put(EntityAttributes.ATTACK_DAMAGE.getId(),
//				new EntityAttributeModifier(MODIFIER_DAMAGE, "Weapon modifier", (double) modifier, 0));
//			multimap.put(EntityAttributes.ATTACK_SPEED.getId(),
//				new EntityAttributeModifier(MODIFIER_SWING_SPEED, "Weapon modifier", -2.4000000953674316D, 0));
//		}
//		return multimap;
//	}

	@Override
	public class_1271<class_1799> method_7836(final class_1937 world, final class_1657 player, final class_1268 hand) {
		final class_1799 stack = player.method_5998(hand);
		if (player.method_5715()) {
			if (new ItemPowerManager(stack).getEnergyStored() < cost) {
				ChatUtils.sendNoSpamMessages(MessageIDs.nanosaberID, new class_2585(
					class_124.field_1080 + StringUtils.t("techreborn.message.nanosaberEnergyErrorTo") + " "
						+ class_124.field_1065 + StringUtils.t("techreborn.message.nanosaberActivate")));
			} else {
				if (!ItemUtils.isActive(stack)) {
					if (stack.method_7969() == null) {
						stack.method_7980(new class_2487());
					}
					stack.method_7969().method_10556("isActive", true);
					if (world.field_9236) {
						ChatUtils.sendNoSpamMessages(MessageIDs.nanosaberID, new class_2585(
							class_124.field_1080 + StringUtils.t("techreborn.message.setTo") + " "
								+ class_124.field_1065 + StringUtils.t("techreborn.message.nanosaberActive")));
					}
				} else {
					stack.method_7969().method_10556("isActive", false);
					if (world.field_9236) {
						ChatUtils.sendNoSpamMessages(MessageIDs.nanosaberID, new class_2585(
							class_124.field_1080 + StringUtils.t("techreborn.message.setTo") + " "
								+ class_124.field_1065 + StringUtils.t("techreborn.message.nanosaberInactive")));
					}
				}
			}
			return new class_1271<>(class_1269.field_5812, stack);
		}
		return new class_1271<>(class_1269.field_5811, stack);
	}

	@Override
	public void method_7888(class_1799 stack, class_1937 worldIn, class_1297 entityIn, int itemSlot, boolean isSelected) {
		if (ItemUtils.isActive(stack) && new ItemPowerManager(stack).getEnergyStored() < cost) {
			if(worldIn.field_9236){
				ChatUtils.sendNoSpamMessages(MessageIDs.nanosaberID, new class_2585(
					class_124.field_1080 + StringUtils.t("techreborn.message.nanosaberEnergyError") + " "
						+ class_124.field_1065 + StringUtils.t("techreborn.message.nanosaberDeactivating")));
			}
			stack.method_7969().method_10556("isActive", false);
		}
	}

	@Override
	public boolean method_7878(class_1799 itemStack_1, class_1799 itemStack_2) {
		return false;
	}

	@Override
	public double getDurability(class_1799 stack) {
		return 1 - ItemUtils.getPowerForDurabilityBar(stack);
	}

	@Override
	public boolean showDurability(class_1799 stack) {
		return true;
	}

	@Override
	public int getDurabilityColor(class_1799 stack) {
		return PowerSystem.getDisplayPower().colour;
	}

	@Environment(EnvType.CLIENT)
	@Override
	public void method_7850(
		class_1761 par2ItemGroup, class_2371<class_1799> itemList) {
		if (!method_7877(par2ItemGroup)) {
			return;
		}
		class_1799 inactiveUncharged = new class_1799(this);
		inactiveUncharged.method_7980(new class_2487());
		inactiveUncharged.method_7969().method_10556("isActive", false);

		class_1799 inactiveCharged = new class_1799(TRContent.NANOSABER);
		inactiveCharged.method_7980(new class_2487());
		inactiveCharged.method_7969().method_10556("isActive", false);
		ItemPowerManager capEnergy = new ItemPowerManager(inactiveCharged);
		capEnergy.setEnergyStored(capEnergy.getMaxEnergyStored());

		class_1799 activeCharged = new class_1799(TRContent.NANOSABER);
		activeCharged.method_7980(new class_2487());
		activeCharged.method_7969().method_10556("isActive", true);
		ItemPowerManager capEnergy2 = new ItemPowerManager(activeCharged);
		capEnergy2.setEnergyStored(capEnergy2.getMaxEnergyStored());

		itemList.add(inactiveUncharged);
		itemList.add(inactiveCharged);
		itemList.add(activeCharged);
	}

	@Environment(EnvType.CLIENT)
	@Override
	public void method_7851(class_1799 stack, @Nullable class_1937 worldIn, List<class_2561> tooltip, class_1836 flagIn) {
		if (!ItemUtils.isActive(stack)) {
			tooltip.add(new class_2588("techreborn.message.nanosaberInactive").method_10854(class_124.field_1080));
		} else {
			tooltip.add(new class_2588("techreborn.message.nanosaberActive").method_10854(class_124.field_1080));
		}
	}

	// IEnergyItemInfo
	@Override
	public int getCapacity() {
		return maxCharge;
	}

	@Override
	public int getMaxInput() {
		return transferLimit;
	}

	@Override
	public int getMaxOutput() {
		return 0;
	}
}
