/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.packets;

import net.fabricmc.fabric.api.network.PacketContext;
import net.fabricmc.fabric.api.network.ServerSidePacketRegistry;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_2338;
import net.minecraft.class_2586;
import net.minecraft.class_2596;
import net.minecraft.class_2960;
import reborncore.common.network.ExtendedPacketBuffer;
import reborncore.common.network.NetworkManager;
import techreborn.blockentity.fusionReactor.FusionControlComputerBlockEntity;
import techreborn.blockentity.machine.tier1.AutoCraftingTableBlockEntity;
import techreborn.blockentity.machine.tier1.RollingMachineBlockEntity;
import techreborn.blockentity.storage.AdjustableSUBlockEntity;
import techreborn.config.TechRebornConfig;
import techreborn.init.TRContent;

import java.util.function.BiConsumer;

public class ServerboundPackets {

	public static final class_2960 AESU = new class_2960("techreborn", "aesu");
	public static final class_2960 AUTO_CRAFTING_LOCK = new class_2960("techreborn", "auto_crafting_lock");
	public static final class_2960 ROLLING_MACHINE_LOCK = new class_2960("techreborn", "rolling_machine_lock");
	public static final class_2960 FUSION_CONTROL_SIZE = new class_2960("techreborn", "fusion_control_size");
	public static final class_2960 REFUND = new class_2960("techreborn", "refund");
	
	public static void init() {
		registerPacketHandler(AESU, (extendedPacketBuffer, context) -> {
			class_2338 pos = extendedPacketBuffer.method_10811();
			int buttonID = extendedPacketBuffer.readInt();
			boolean shift = extendedPacketBuffer.readBoolean();
			boolean ctrl = extendedPacketBuffer.readBoolean();

			context.getTaskQueue().execute(() -> {
				class_2586 blockEntity = context.getPlayer().field_6002.method_8321(pos);
				if (blockEntity instanceof AdjustableSUBlockEntity) {
					((AdjustableSUBlockEntity) blockEntity).handleGuiInputFromClient(buttonID, shift, ctrl);
				}
			});
		});

		registerPacketHandler(AUTO_CRAFTING_LOCK, (extendedPacketBuffer, context) -> {
			class_2338 machinePos = extendedPacketBuffer.method_10811();
			boolean locked = extendedPacketBuffer.readBoolean();

			context.getTaskQueue().execute(() -> {
				class_2586 BlockEntity = context.getPlayer().field_6002.method_8321(machinePos);
				if (BlockEntity instanceof AutoCraftingTableBlockEntity) {
					((AutoCraftingTableBlockEntity) BlockEntity).locked = locked;
				}
			});
		});

		registerPacketHandler(FUSION_CONTROL_SIZE, (extendedPacketBuffer, context) -> {
			int sizeDelta = extendedPacketBuffer.readInt();
			class_2338 pos = extendedPacketBuffer.method_10811();

			context.getTaskQueue().execute(() -> {
				class_2586 blockEntity = context.getPlayer().field_6002.method_8321(pos);
				if (blockEntity instanceof FusionControlComputerBlockEntity) {
					((FusionControlComputerBlockEntity) blockEntity).changeSize(sizeDelta);
				}
			});
		});

		registerPacketHandler(ROLLING_MACHINE_LOCK, (extendedPacketBuffer, context) -> {
			class_2338 machinePos = extendedPacketBuffer.method_10811();
			boolean locked = extendedPacketBuffer.readBoolean();

			context.getTaskQueue().execute(() -> {
				class_2586 BlockEntity = context.getPlayer().field_6002.method_8321(machinePos);
				if (BlockEntity instanceof RollingMachineBlockEntity) {
					((RollingMachineBlockEntity) BlockEntity).locked = locked;
				}
			});
		});

		registerPacketHandler(REFUND, (extendedPacketBuffer, context) -> {
			if(!TechRebornConfig.allowManualRefund){
				return;
			}
			context.getTaskQueue().execute(() -> {
				class_1657 playerMP = context.getPlayer();
				for (int i = 0; i < playerMP.field_7514.method_5439(); i++) {
					class_1799 stack = playerMP.field_7514.method_5438(i);
					if (stack.method_7909() == TRContent.MANUAL) {
						playerMP.field_7514.method_5441(i);
						playerMP.field_7514.method_7394(new class_1799(class_1802.field_8529));
						playerMP.field_7514.method_7394(TRContent.Ingots.REFINED_IRON.getStack());
						return;
					}
				}
			});

		});
	}

	private static void registerPacketHandler(class_2960 identifier, BiConsumer<ExtendedPacketBuffer, PacketContext> consumer){
		ServerSidePacketRegistry.INSTANCE.register(identifier, (packetContext, packetByteBuf) -> consumer.accept(new ExtendedPacketBuffer(packetByteBuf), packetContext));
	}

	public static class_2596 createPacketAesu(int buttonID, boolean shift, boolean ctrl, AdjustableSUBlockEntity blockEntity) {
		return NetworkManager.createServerBoundPacket(AESU, extendedPacketBuffer -> {
			extendedPacketBuffer.method_10807(blockEntity.method_11016());
			extendedPacketBuffer.writeInt(buttonID);
			extendedPacketBuffer.writeBoolean(shift);
			extendedPacketBuffer.writeBoolean(ctrl);
		});
	}

	public static class_2596 createPacketAutoCraftingTableLock(AutoCraftingTableBlockEntity machine, boolean locked) {
		return NetworkManager.createServerBoundPacket(AUTO_CRAFTING_LOCK, extendedPacketBuffer -> {
			extendedPacketBuffer.method_10807(machine.method_11016());
			extendedPacketBuffer.writeBoolean(locked);
		});
	}

	public static class_2596 createPacketFusionControlSize(int sizeDelta, class_2338 pos) {
		return NetworkManager.createServerBoundPacket(FUSION_CONTROL_SIZE, extendedPacketBuffer -> {
			extendedPacketBuffer.writeInt(sizeDelta);
			extendedPacketBuffer.method_10807(pos);
		});
	}


	public static class_2596 createPacketRollingMachineLock(RollingMachineBlockEntity machine, boolean locked) {
		return NetworkManager.createServerBoundPacket(ROLLING_MACHINE_LOCK, extendedPacketBuffer -> {
			extendedPacketBuffer.method_10807(machine.method_11016());
			extendedPacketBuffer.writeBoolean(locked);
		});
	}

	public static class_2596 createRefundPacket(){
		return NetworkManager.createServerBoundPacket(REFUND, extendedPacketBuffer -> {

		});
	}

}
