/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.world;

import java.util.Random;
import java.util.Set;
import java.util.function.Function;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2680;
import net.minecraft.class_2944;
import net.minecraft.class_3111;
import net.minecraft.class_3341;
import net.minecraft.class_3747;
import com.mojang.datafixers.Dynamic;
import techreborn.blocks.misc.BlockRubberLog;
import techreborn.init.TRContent;

/**
 * @author drcrazy
 *
 */
public class RubberTreeFeature extends class_2944<class_3111> {

	// TODO: Configs
	private int treeBaseHeight = 5;
	private int sapRarity = 10;
	private int spireHeight = 4;
	

	public RubberTreeFeature(Function<Dynamic<?>, ? extends class_3111> featureConfig, boolean notify) {
		super(featureConfig, notify);
	}

	@Override
	protected boolean method_12775(Set<class_2338> changedBlocks, class_3747 worldIn, Random rand,
			class_2338 saplingPos, class_3341 mutableBoundingBox) {
		int treeHeight = rand.nextInt(5) + treeBaseHeight;
		int baseY = saplingPos.method_10264();
		int baseX = saplingPos.method_10263();
		int baseZ = saplingPos.method_10260();
		if (baseY <= 1 && baseY + treeHeight + spireHeight + 1 >= 256) {
			return false;
		}

		if (!method_16433(worldIn, saplingPos.method_10074())) {
			return false;
		}

		int yOffset;
		int xOffset;
		int zOffset;
		for (yOffset = baseY; yOffset <= baseY + 1 + treeHeight; ++yOffset) {
			byte radius = 1;
			if (yOffset == baseY) {
				radius = 0;
			}
			if (yOffset >= baseY + 1 + treeHeight - 2) {
				radius = 2;
			}

			class_2338.class_2339 blockpos$mutable = new class_2338.class_2339();
			for (xOffset = baseX - radius; xOffset <= baseX + radius; ++xOffset) {
				for (zOffset = baseZ - radius; zOffset <= baseZ + radius; ++zOffset) {
					if (!method_16420(worldIn, blockpos$mutable.method_10103(xOffset, yOffset, zOffset))) {
						return false;
					}
				}
			}
		}

		// Ok, we are cleared for take off!
		this.method_16427(worldIn, saplingPos.method_10074());

		// Leaves
		class_2680 leafState = TRContent.RUBBER_LEAVES.method_9564();
		for (yOffset = baseY - 3 + treeHeight; yOffset <= baseY + treeHeight; ++yOffset) {
			int var12 = yOffset - (baseY + treeHeight), center = 1 - var12 / 2;
			for (xOffset = baseX - center; xOffset <= baseX + center; ++xOffset) {
				int xPos = xOffset - baseX, t = xPos >> 15;
				xPos = (xPos + t) ^ t;
				for (zOffset = baseZ - center; zOffset <= baseZ + center; ++zOffset) {
					int zPos = zOffset - baseZ;
					zPos = (zPos + (t = zPos >> 31)) ^ t;
					if ((xPos != center | zPos != center) || rand.nextInt(2) != 0 && var12 != 0) {
						class_2338 leafPos = new class_2338(xOffset, yOffset, zOffset);
						if (method_16420(worldIn, leafPos) || method_16425(worldIn, leafPos)) {
							this.method_12773(changedBlocks, worldIn, leafPos, leafState, mutableBoundingBox);
						}
					}
				}
			}
		}

		// Trunk
		class_2338 currentLogPos = null;
		class_2680 logState = TRContent.RUBBER_LOG.method_9564();
		for (yOffset = 0; yOffset < treeHeight; ++yOffset) {
			currentLogPos = saplingPos.method_10086(yOffset);
			if (!method_16420(worldIn, currentLogPos)) {
				continue;
			}
			if (rand.nextInt(sapRarity) == 0) {
				logState = logState.method_11657(BlockRubberLog.HAS_SAP, true).method_11657(BlockRubberLog.SAP_SIDE,
						class_2350.method_10139(rand.nextInt(4)));
			}
			this.method_12773(changedBlocks, worldIn, currentLogPos, logState, mutableBoundingBox);
		}

		// Spire
		if (currentLogPos != null) {
			for (int i = 0; i < spireHeight; i++) {
				class_2338 blockpos = currentLogPos.method_10086(i);
				this.method_12773(changedBlocks, worldIn, blockpos, leafState, mutableBoundingBox);
			}
		}

		return true;
	}

}
